#include "sprite_loader.h"

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>

SDL_Surface* loadSDLImageSprite(const char* spritePath)
{
    SDL_Surface* pSprite = IMG_Load(spritePath);
    if ( !pSprite )
    {
        fprintf(stderr,"Fail to load sprite '%s' -> '%s'\n",spritePath,IMG_GetError());
    }
    
    return pSprite;
}

SDL_Surface* loadSDLImageSpriteOptimised(const char* spritePath)
{
    SDL_Color transparencyColour = {0, 0, 0, 0};
    
    SDL_Surface* pSurface = IMG_Load(spritePath);
    if ( pSurface != NULL )
    {
        // Nous optimisons la texture pour correspondre avec le format de l'écran
        SDL_Surface* pOptimisedSurface = SDL_DisplayFormat(pSurface);
        if ( pOptimisedSurface != NULL )
        {
            // Nous libérons la mémoire associée à l'ancienne surface
            SDL_FreeSurface(pSurface);

            // Nous définissons une couleur de transparence
            Uint32 colorkey = SDL_MapRGB(pOptimisedSurface->format, transparencyColour.r, transparencyColour.g, transparencyColour.b);
            if ( SDL_SetColorKey(pOptimisedSurface, SDL_RLEACCEL, colorkey ) == -1 )
            {
                fprintf(stderr,"Fail to set transparency in '%s'\n",spritePath);
            }

            // Nous remplaçons notre pointeur final par le pointeur sur la surface optimisée
            pSurface = pOptimisedSurface;
        }
        else
        {
            fprintf(stderr,"Fail to optimise sprite '%s'\n",spritePath);
        }
    }
    else
    {
        fprintf(stderr,"Fail to load sprite '%s'\n",spritePath);
    }
    
    return pSurface;
}

SDL_Surface* loadBMPSprite(const char* spritePath)
{
    SDL_Surface* pSurface = SDL_LoadBMP(spritePath);
    if ( pSurface == NULL )
    {
        fprintf(stderr,"Fail to load sprite '%s'\n",spritePath);
    }
    
    return pSurface;
}

SDL_Surface* loadBMPSpriteOptimised(const char* spritePath)
{
    SDL_Color transparencyColour = {0, 0, 0, 0};
    
    SDL_Surface* pSurface = SDL_LoadBMP(spritePath);
    if ( pSurface != NULL )
    {
        // Nous optimisons la texture pour correspondre avec le format de l'écran
        SDL_Surface* pOptimisedSurface = SDL_DisplayFormat(pSurface);
        if ( pOptimisedSurface != NULL )
        {
            // Nous libérons la mémoire associée à l'ancienne surface
            SDL_FreeSurface(pSurface);

            // Nous définissons une couleur de transparence
            Uint32 colorkey = SDL_MapRGB(pOptimisedSurface->format, transparencyColour.r, transparencyColour.g, transparencyColour.b);
            if ( SDL_SetColorKey(pOptimisedSurface, SDL_RLEACCEL, colorkey ) == -1 )
            {
                fprintf(stderr,"Fail to set transparency in '%s'\n",spritePath);
            }

            // Nous remplaçons notre pointeur final par le pointeur sur la surface optimisée
            pSurface = pOptimisedSurface;
        }
        else
        {
            fprintf(stderr,"Fail to optimise sprite '%s'\n",spritePath);
        }
    }
    else
    {
        fprintf(stderr,"Fail to load sprite '%s'\n",spritePath);
    }
    
    return pSurface;
}


SDL_Surface* loadBMPSpriteColorKey(const char* spritePath)
{
    SDL_Color transparencyColour = {255, 0, 255, 255};
    
    SDL_Surface* pSurface = SDL_LoadBMP(spritePath);
    if ( pSurface != NULL )
    {
        // Nous définissons une couleur de transparence
        Uint32 colorkey = SDL_MapRGB(pSurface->format, transparencyColour.r, transparencyColour.g, transparencyColour.b);
        if ( SDL_SetColorKey(pSurface, SDL_SRCCOLORKEY, colorkey ) == -1 )
        {
            fprintf(stderr,"Fail to set transparency in '%s'\n",spritePath);
        }
    }
    else
    {
        fprintf(stderr,"Fail to load sprite '%s'\n",spritePath);
    }
    
    return pSurface;
}


SDL_Surface* loadBMPSpriteColorKeyOptimised(const char* spritePath)
{
    SDL_Color transparencyColour = {255, 0, 255, 255};
    
    SDL_Surface* pSurface = SDL_LoadBMP(spritePath);
    if ( pSurface != NULL )
    {
        // Nous optimisons la texture pour correspondre avec le format de l'écran
        SDL_Surface* pOptimisedSurface = SDL_DisplayFormat(pSurface);
        if ( pOptimisedSurface != NULL )
        {
            // Nous libérons la mémoire associée à l'ancienne surface
            SDL_FreeSurface(pSurface);

            // Nous définissons une couleur de transparence
            Uint32 colorkey = SDL_MapRGB(pOptimisedSurface->format, transparencyColour.r, transparencyColour.g, transparencyColour.b);
            if ( SDL_SetColorKey(pOptimisedSurface, SDL_RLEACCEL | SDL_SRCCOLORKEY, colorkey ) == -1 )
            {
                fprintf(stderr,"Fail to set transparency in '%s'\n",spritePath);
            }

            // Nous remplaçons notre pointeur final par le pointeur sur la surface optimisée
            pSurface = pOptimisedSurface;
        }
        else
        {
            fprintf(stderr,"Fail to optimise sprite '%s'\n",spritePath);
        }
    }
    else
    {
        fprintf(stderr,"Fail to load sprite '%s'\n",spritePath);
    }
    
    return pSurface;
}
