/**
 * Projet Code::Blocks minimaliste afin de donner un projet pre configure et pret  fonctionner
 * @author Alexandre "LittleWhite" Laurent
 */

#include <stdio.h>

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>

#ifndef WIDTH
    #define WIDTH 1366
#endif

#ifndef HEIGHT
    #define HEIGHT 768
#endif

#ifndef NB_ITERATION
    #define NB_ITERATION 50
#endif

#define DO_STR(x) #x
#define STR(x) DO_STR(x)

#define STR_WIDTH STR(WIDTH)
#define STR_HEIGHT STR(HEIGHT)
#define STR_NB_ITERATION STR(NB_ITERATION)

const char* resultFileName = "./results/test2_"STR_WIDTH"_"STR_HEIGHT"_"STR_NB_ITERATION".txt";

typedef struct FrameData
{
    unsigned int nbFrames;
}FrameData;

int main()
{
    SDL_Surface* pWindowSurface = NULL; // Ecran principal
    char mustContinue = 1; // Indicateur boolean pour la boucle principale
    unsigned int lastUpdate = 0;
    unsigned int nbFrames = 0;
    
    FrameData results[NB_ITERATION] = { {0} };
    unsigned int resultsCounter=0;

    // Demarre SDL
    if ( SDL_Init(SDL_INIT_VIDEO) == -1 )
    {
        fprintf(stderr,"Erreur lors de l'initialisation de la SDL\n");
        return -1;
    }

    // Creer la fenetre
    pWindowSurface = SDL_SetVideoMode(WIDTH,HEIGHT,32,SDL_SWSURFACE | SDL_DOUBLEBUF);
	if ( pWindowSurface == NULL )
	{
	    fprintf(stderr,"Erreur lors de l'ouverture de la fenetre\n");
	    SDL_Quit();
        return -2;
	}
    
    // Boucle principale
	while ( mustContinue && resultsCounter < NB_ITERATION )
	{
	    SDL_Event event;
	    SDL_PollEvent(&event);

        // Detecte si on clique sur la croix
	    if( event.type == SDL_QUIT )
	    {
            //On quitte le programme
            mustContinue = 0;
		}
		// Detecte si une touche est appuyee
		else if( event.type == SDL_KEYDOWN )
		{
            if ( event.key.keysym.sym == SDLK_ESCAPE )
            {
                mustContinue = 0;
            }
		}
        
        SDL_Flip(pWindowSurface);
        
        nbFrames++;
        if ( SDL_GetTicks()-lastUpdate > 1000 )
        {
            results[resultsCounter].nbFrames = nbFrames;
            nbFrames = 0;
            lastUpdate+=1000;
            
            resultsCounter++;
        }
	}

    SDL_Quit();
    
    // Ecrit les resultats
    if ( resultsCounter != 0 )
    {
        unsigned int avgNBFrames = 0;
        unsigned int i = 0;
        
        FILE* pResultsFile = fopen(resultFileName,"w");
        if ( pResultsFile == NULL )
        {
            fprintf(stderr,"Fail to open '%s'\n",resultFileName);
            return -1;
        }
        
        for ( i = 0 ; i < resultsCounter ; i++ )
        {
            fprintf(pResultsFile,"%u\n",results[i].nbFrames);
            avgNBFrames += results[i].nbFrames;
        }
        avgNBFrames/=resultsCounter;
        fprintf(pResultsFile,"Average : %u on %u\n",avgNBFrames,resultsCounter);
        
        fclose(pResultsFile);
    }

    return 0;
}
